sys class final

sys

Inheritance
  • Object
  • PythonObjectInterface<PythonFfiDelegate<Object?>, Object?>
  • PythonObject
  • PythonModule
  • sys

Constructors

sys.from(PythonModuleInterface<PythonFfiDelegate<Object?>, Object?> pythonModule)

Properties

$platform ↔ Object?
platform (getter)
getter/setter pair
abiflags ↔ Object?
abiflags (getter)
getter/setter pair
api_version ↔ Object?
api_version (getter)
getter/setter pair
argv ↔ Object?
argv (getter)
getter/setter pair
base_exec_prefix ↔ Object?
base_exec_prefix (getter)
getter/setter pair
base_prefix ↔ Object?
base_prefix (getter)
getter/setter pair
builtin_module_names ↔ Object?
builtin_module_names (getter)
getter/setter pair
byteorder ↔ Object?
byteorder (getter)
getter/setter pair
copyright (getter)
getter/setter pair
dont_write_bytecode ↔ Object?
dont_write_bytecode (getter)
getter/setter pair
exec_prefix ↔ Object?
exec_prefix (getter)
getter/setter pair
executable ↔ Object?
executable (getter)
getter/setter pair
finalizer → Finalizer<(PythonFfiDelegate<Object?>, Object?)>
The finalizer for the python object. Gets invoked when the object is no longer accessible to the program.
finalinherited
flags ↔ Object?
flags (getter)
getter/setter pair
float_info ↔ Object?
float_info (getter)
getter/setter pair
float_repr_style ↔ Object?
float_repr_style (getter)
getter/setter pair
hash_info ↔ Object?
hash_info (getter)
getter/setter pair
hashCode → int
The hash code for this object.
no setterinherited
hexversion ↔ Object?
hexversion (getter)
getter/setter pair
initializer → Initializer<PythonFfiDelegate<Object?>, Object?>
The initializer for the python object. Gets invoked at the start of the constructor.
finalinherited
int_info ↔ Object?
int_info (getter)
getter/setter pair
maxsize ↔ Object?
maxsize (getter)
getter/setter pair
maxunicode ↔ Object?
maxunicode (getter)
getter/setter pair
meta_path ↔ Object?
meta_path (getter)
getter/setter pair
modules ↔ Object?
modules (getter)
getter/setter pair
orig_argv ↔ Object?
orig_argv (getter)
getter/setter pair
path ↔ Object?
path (getter)
getter/setter pair
path_hooks ↔ Object?
path_hooks (getter)
getter/setter pair
path_importer_cache ↔ Object?
path_importer_cache (getter)
getter/setter pair
platform → PythonFfiDelegate<Object?>
Gets the platform that this object is associated with.
no setterinherited
platlibdir ↔ Object?
platlibdir (getter)
getter/setter pair
prefix ↔ Object?
prefix (getter)
getter/setter pair
pycache_prefix ↔ Null
pycache_prefix (getter)
getter/setter pair
reference → Object?
Gets the reference to the python object.
no setterinherited
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited
thread_info ↔ Object?
thread_info (getter)
getter/setter pair
version ↔ Object?
version (getter)
getter/setter pair
version_info ↔ Object?
version_info (getter)
getter/setter pair
warnoptions ↔ Object?
warnoptions (getter)
getter/setter pair

Methods

debugDump() → void
Dumps the python object to the console.
inherited
getAttribute<T extends Object?>(String attributeName) → T
Gets the attribute with the given name.
inherited
getAttributeOrNull<T extends Object?>(String attributeName) → T?
Gets the attribute with the given name or null if it does not exist.
inherited
getAttributeRaw<T extends PythonObjectInterface<PythonFfiDelegate<Object?>, Object?>>(String attributeName) → T
Gets the attribute with the given name.
inherited
getClass(String className, List<Object?> args, [Map<String, Object?>? kwargs]) → PythonClassInterface<PythonFfiDelegate<Object?>, Object?>
Gets a class from the module.
inherited
getFunction(String name) → PythonFunctionInterface<PythonFfiDelegate<Object?>, Object?>
Gets the function with the given name.
inherited
hasAttribute(String attributeName) → bool
Checks if the python object has the given attribute.
inherited
noSuchMethod(Invocation invocation) → Object?
Invoked when a nonexistent method or property is accessed.
inherited
setAttribute<T extends Object?>(String attributeName, T value) → void
Sets the attribute with the given name.
inherited
setAttributeRaw<T extends PythonObjectInterface<PythonFfiDelegate<Object?>, Object?>>(String attributeName, T value) → void
Sets the attribute with the given name.
inherited
toDartObject() → Object?
Converts the python object to a Dart object.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited

Static Methods

import() → sys
Imports a Python module.
override