dartpip_solver library
Version solver used by the dartpip command to resolve python modules.
Classes
- Constraint
- Abstraction of a constraint on a Python dependency. This will be turned into a Dependency by solve.
- Dependency
- Abstraction of a Python dependency resolved to a specific version.
-
LazyFuture<
T> - A Future that is only evaluated once.
- PyPIClient
- A client for interacting with the the PyPI-API.
- PyPIService
- A collection of functions for interacting with PyPI.
Extensions
- NameExtension on FileSystemEntity
- Adds a 'name' getter to each FileSystemEntity.
Constants
- kDartpipSolverVersion → const String
- The version of dartpip_solver.
Functions
-
solve(
Iterable< Constraint> constraints, {bool? verboseLogging}) → Future<Set< Dependency> > - Solves a set of constraints on Python dependencies. Returns a set of dependencies that represent the resolved dependencies.